@@ -220,12 +220,7 @@ proc lbots {} {
 }
 
 proc iso {nick chan1} {
- if {[matchattr [nick2hand $nick $chan1] o] || 
- [matchchanattr [nick2hand $nick $chan1] o $chan1]} {
-  return 1
-  break
- }
- return 0
+ return [matchattr [nick2hand $nick $chan1] o|o $chan1]
 }
 
 proc validchan {chan} {
